/*
 This file is part of the digitalSTROM ds485-core library.

 Copyright (C) 2010 digitalSTROM.org, Schlieren, Switzerland

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation version 2
 of the License.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301, USA.
 */

#include    <iomanip>
#include    <iostream>
#include    <sstream>

#include "digitalSTROM/ds485-const.h"
#include <string>

using namespace std;

#define     fdRead      0
#define     fdWrite     1

std::string decode_command(int command) {
    switch (command) {
    case 0:
        return "TOKEN";
    case 1:
        return "STARTUP";
    case 2:
        return "JOIN";
    case 3:
        return "GETADDR";
    case 4:
        return "SETADDR";
    case 5:
        return "SETSUCC";
    case 6:
        return "RESP";
    case 8:
        return "RESTART";
    }
    return "?";
}
std::string decode_type(int type) {
    switch (type) {
    case 0:
        return "REQ";
    case 1:
        return "BSY";
    case 2:
        return "ACK";
    case 3:
        return "EVT";
    }
    return "?";
}

int main(int argc, char** argv)
{
  unsigned char     bytes[200];
  unsigned int      data;
  unsigned int      idx;
  string            line; 

  

  while( getline(cin, line) )
  { 
      string    timestamp;
      idx   = 0;
      
      size_t  pos = line.find( ';' );
      
      if( pos != string::npos )
      {
        timestamp = line.substr( 0, pos );
        line.erase( 0, pos+1 );
      } else {
          continue;
      }
      
//      cout << pos << endl;
//      cout << timestamp << endl;
//      cout << line << endl;
      
      stringstream  strm( line );
      
      while( strm >> std::hex >> data )
      {
        bytes[idx]  = data;
        idx++;
        if( idx >= sizeof(bytes) / sizeof(bytes[0]) )
        {
          return    -1;
        }
      }
      
      if (bytes[idx-1] == 0xFF) {
          --idx;
          if (bytes[idx-1] == 0xFF) {
              --idx;
          }
      }

      int dest = bytes[0] & ~DS485_DESTINATION_MASK;
      bool data = (bytes[0] & DS485_DATACONTROL_MASK) == DS485_DATACONTROL_MASK;
      bool broadcast = (bytes[0] & DS485_BROADCAST_MASK) == DS485_BROADCAST_MASK;
      int source = bytes[1] & ~DS485_SOURCE_MASK;
      int type = bytes[1] >> 6;
      bool request = (bytes[1] & DS485_SOURCE_MASK) == DS485_REQ_MASK;
      bool busy = (bytes[1] & DS485_SOURCE_MASK) == DS485_BUSY_MASK;
      bool ack_resp = (bytes[1] & DS485_SOURCE_MASK) == DS485_ACK_RESPONSE_MASK;
      bool event = (bytes[1] & DS485_SOURCE_MASK) == DS485_EVENT_MASK;
      int command = bytes[2] >> 4;
      int length = bytes[2] & DS485_LENGTH_MASK;

//      if( (bytes[0] & 0x80) == 0 )    // not a data packet
//      {
//        continue;
//      }
      
      if( pos != string::npos )
      {
        cout << timestamp << "; ";
      }
      
      cout << setfill(' ');

      cout << source << "->";
      if (broadcast) {
          cout << "*";
      } else {
          cout  << dest;
      }
      cout << " ";
      if (!data) {
          cout << decode_command(command);
          if (ack_resp) {
              cout << " RESP";
          }
      } else {
          if (request) {
              cout << "req";
          } else if (ack_resp) {
              cout << "resp";
          } else if (event) {
              cout << "event";
          } else {
              cout << " ";
          }
      }
      cout << "\t";

//      cout << setw(2) << hex << (bytes[1] && 0x3f);       // Source
//      cout << "->";
//      cout << setw(2) << hex << (bytes[0] && 0x3f);       // Destination
//      cout << " ";
//      cout << setw(2) << hex << (bytes[2] >> 4);           // Command
//      cout << " ";
//      cout << setw(2) << hex << (bytes[2] & 0x0f);     // Length
//      cout << " ";
      
      cout << setfill('0');
      int data_len = length + 3;
      if (3+length > idx){
          cout << "<short data> ";
          data_len = idx;
      }
      for (unsigned int i = 3; i < data_len; i++)
      {
        cout << setw(2) << hex << (int)bytes[i];
        cout << " ";
      }

      cout << endl;
  }
  
  return    0;
}

